@@ -2,10 +2,11 @@ package css
 
 import (
 	"bytes"
-	"io"
+	"fmt"
 	"strconv"
 
 	"github.com/tdewolff/parse/v2"
+	"github.com/tdewolff/parse/v2/buffer"
 )
 
 var wsBytes = []byte(" ")
@@ -76,15 +77,16 @@ func (t Token) String() string {
 
 // Parser is the state for the parser.
 type Parser struct {
-	l     *Lexer
-	state []State
-	err   error
+	l      *Lexer
+	state  []State
+	err    string
+	errPos int
 
 	buf   []Token
 	level int
 
-	tt          TokenType
 	data        []byte
+	tt          TokenType
 	keepWS      bool
 	prevWS      bool
 	prevEnd     bool
@@ -92,7 +94,7 @@ type Parser struct {
 }
 
 // NewParser returns a new CSS parser from an io.Reader. isInline specifies whether this is an inline style attribute.
-func NewParser(r io.Reader, isInline bool) *Parser {
+func NewParser(r *parse.Input, isInline bool) *Parser {
 	l := NewLexer(r)
 	p := &Parser{
 		l:     l,
@@ -107,27 +109,23 @@ func NewParser(r io.Reader, isInline bool) *Parser {
 	return p
 }
 
+// HasParseError returns true if there is a parse error (and not a read error).
+func (p *Parser) HasParseError() bool {
+	return p.err != ""
+}
+
 // Err returns the error encountered during parsing, this is often io.EOF but also other errors can be returned.
 func (p *Parser) Err() error {
-	if p.err != nil {
-		return p.err
+	if p.err != "" {
+		r := buffer.NewReader(p.l.r.Bytes())
+		return parse.NewError(r, p.errPos, p.err)
 	}
 	return p.l.Err()
 }
 
-// Restore restores the NULL byte at the end of the buffer.
-func (p *Parser) Restore() {
-	p.l.Restore()
-}
-
-// Offset returns the current position in the input stream.
-func (p *Parser) Offset() int {
-	return p.l.Offset()
-}
-
 // Next returns the next Grammar. It returns ErrorGrammar when an error was encountered. Using Err() one can retrieve the error message.
 func (p *Parser) Next() (GrammarType, TokenType, []byte) {
-	p.err = nil
+	p.err = ""
 
 	if p.prevEnd {
 		p.tt, p.data = RightBraceToken, endBytes
@@ -213,12 +211,12 @@ func (p *Parser) parseDeclarationList() GrammarType {
 	// parse error
 	p.initBuf()
 	p.l.r.Move(-len(p.data))
-	p.err = parse.NewErrorLexer(p.l.r, "CSS parse error: unexpected token '%s' in declaration", string(p.data))
+	p.err, p.errPos = fmt.Sprintf("CSS parse error: unexpected token '%s' in declaration", string(p.data)), p.l.r.Offset()
 	p.l.r.Move(len(p.data))
 
 	if p.tt == RightBraceToken {
 		// right brace token will occur when we've had a decl error that ended in a right brace token
-		// as these are not handled by decl error, we handle it here explictly. Normally its used to end eg. the qual rule.
+		// as these are not handled by decl error, we handle it here explicitly. Normally its used to end eg. the qual rule.
 		p.pushBuf(p.tt, p.data)
 		return ErrorGrammar
 	}
@@ -336,7 +334,7 @@ func (p *Parser) parseQualifiedRule() GrammarType {
 			p.state = append(p.state, (*Parser).parseQualifiedRuleDeclarationList)
 			return BeginRulesetGrammar
 		} else if tt == ErrorToken {
-			p.err = parse.NewErrorLexer(p.l.r, "CSS parse error: unexpected ending in qualified rule")
+			p.err, p.errPos = "CSS parse error: unexpected ending in qualified rule", p.l.r.Offset()
 			return ErrorGrammar
 		} else if tt == LeftParenthesisToken || tt == LeftBraceToken || tt == LeftBracketToken || tt == FunctionToken {
 			p.level++
@@ -381,7 +379,7 @@ func (p *Parser) parseDeclaration() GrammarType {
 	tt, data := p.popToken(false)
 	if tt != ColonToken {
 		p.l.r.Move(-len(data))
-		p.err = parse.NewErrorLexer(p.l.r, "CSS parse error: expected colon in declaration")
+		p.err, p.errPos = "CSS parse error: expected colon in declaration", p.l.r.Offset()
 		p.l.r.Move(len(data))
 		p.pushBuf(ttName, dataName)
 		return p.parseDeclarationError(tt, data)
@@ -438,7 +436,7 @@ func (p *Parser) parseCustomProperty() GrammarType {
 	p.initBuf()
 	if tt, data := p.popToken(false); tt != ColonToken {
 		p.l.r.Move(-len(data))
-		p.err = parse.NewErrorLexer(p.l.r, "CSS parse error: expected colon in custom property")
+		p.err, p.errPos = "CSS parse error: expected colon in custom property", p.l.r.Offset()
 		p.l.r.Move(len(data))
 		return ErrorGrammar
 	}
