@@ -6,6 +6,8 @@ import (
 	"strconv"
 )
 
+var ErrInvalidJSON = fmt.Errorf("invalid JSON")
+
 // AST is the full ECMAScript abstract syntax tree.
 type AST struct {
 	Comments  [][]byte // first comments in file
@@ -84,6 +86,11 @@ func (v Var) JS() string {
 	return v.String()
 }
 
+// JSON converts the node back to valid JSON
+func (n Var) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // VarsByUses is sortable by uses in descending order.
 type VarsByUses VarArray
 
@@ -329,6 +336,7 @@ func (s *Scope) Unscope() {
 type INode interface {
 	String() string
 	JS() string
+	JSON() (string, error)
 }
 
 // IStmt is a dummy interface for statements.
@@ -380,6 +388,14 @@ func (n BlockStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n BlockStmt) JSON() (string, error) {
+	if len(n.List) != 1 {
+		return "", ErrInvalidJSON
+	}
+	return n.List[0].JSON()
+}
+
 // EmptyStmt is an empty statement.
 type EmptyStmt struct {
 }
@@ -393,6 +409,11 @@ func (n EmptyStmt) JS() string {
 	return ";"
 }
 
+// JSON converts the node back to valid JSON
+func (n EmptyStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ExprStmt is an expression statement.
 type ExprStmt struct {
 	Value IExpr
@@ -411,6 +432,11 @@ func (n ExprStmt) JS() string {
 	return n.Value.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n ExprStmt) JSON() (string, error) {
+	return n.Value.JSON()
+}
+
 // IfStmt is an if statement.
 type IfStmt struct {
 	Cond IExpr
@@ -446,6 +472,11 @@ func (n IfStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n IfStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // DoWhileStmt is a do-while iteration statement.
 type DoWhileStmt struct {
 	Cond IExpr
@@ -468,6 +499,11 @@ func (n DoWhileStmt) JS() string {
 	return s + " while (" + n.Cond.JS() + ")"
 }
 
+// JSON converts the node back to valid JSON
+func (n DoWhileStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // WhileStmt is a while iteration statement.
 type WhileStmt struct {
 	Cond IExpr
@@ -487,6 +523,11 @@ func (n WhileStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n WhileStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ForStmt is a regular for iteration statement.
 type ForStmt struct {
 	Init IExpr // can be nil
@@ -530,6 +571,11 @@ func (n ForStmt) JS() string {
 	return s + ") " + n.Body.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n ForStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ForInStmt is a for-in iteration statement.
 type ForInStmt struct {
 	Init  IExpr
@@ -546,6 +592,11 @@ func (n ForInStmt) JS() string {
 	return "for (" + n.Init.JS() + " in " + n.Value.JS() + ") " + n.Body.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n ForInStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ForOfStmt is a for-of iteration statement.
 type ForOfStmt struct {
 	Await bool
@@ -571,6 +622,11 @@ func (n ForOfStmt) JS() string {
 	return s + " (" + n.Init.JS() + " of " + n.Value.JS() + ") " + n.Body.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n ForOfStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // CaseClause is a case clause or default clause for a switch statement.
 type CaseClause struct {
 	TokenType
@@ -604,6 +660,11 @@ func (n CaseClause) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n CaseClause) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // SwitchStmt is a switch statement.
 type SwitchStmt struct {
 	Init IExpr
@@ -628,6 +689,11 @@ func (n SwitchStmt) JS() string {
 	return s + " }"
 }
 
+// JSON converts the node back to valid JSON
+func (n SwitchStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // BranchStmt is a continue or break statement.
 type BranchStmt struct {
 	Type  TokenType
@@ -651,6 +717,11 @@ func (n BranchStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n BranchStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ReturnStmt is a return statement.
 type ReturnStmt struct {
 	Value IExpr // can be nil
@@ -673,6 +744,11 @@ func (n ReturnStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n ReturnStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // WithStmt is a with statement.
 type WithStmt struct {
 	Cond IExpr
@@ -688,6 +764,11 @@ func (n WithStmt) JS() string {
 	return "with (" + n.Cond.JS() + ") " + n.Body.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n WithStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // LabelledStmt is a labelled statement.
 type LabelledStmt struct {
 	Label []byte
@@ -703,6 +784,11 @@ func (n LabelledStmt) JS() string {
 	return string(n.Label) + ": " + n.Value.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n LabelledStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ThrowStmt is a throw statement.
 type ThrowStmt struct {
 	Value IExpr
@@ -717,6 +803,11 @@ func (n ThrowStmt) JS() string {
 	return "throw " + n.Value.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n ThrowStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // TryStmt is a try statement.
 type TryStmt struct {
 	Body    *BlockStmt
@@ -756,6 +847,11 @@ func (n TryStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n TryStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // DebuggerStmt is a debugger statement.
 type DebuggerStmt struct {
 }
@@ -769,6 +865,11 @@ func (n DebuggerStmt) JS() string {
 	return "debugger"
 }
 
+// JSON converts the node back to valid JSON
+func (n DebuggerStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // Alias is a name space import or import/export specifier for import/export statements.
 type Alias struct {
 	Name    []byte // can be nil
@@ -788,8 +889,9 @@ func (alias Alias) JS() string {
 	return alias.String()
 }
 
-func (alias *Alias) isNil() bool {
-	return alias == nil
+// JSON converts the node back to valid JSON
+func (n Alias) JSON() (string, error) {
+	return "", ErrInvalidJSON
 }
 
 // ImportStmt is an import statement.
@@ -856,6 +958,11 @@ func (n ImportStmt) JS() string {
 	return s + " " + string(n.Module)
 }
 
+// JSON converts the node back to valid JSON
+func (n ImportStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ExportStmt is an export statement.
 type ExportStmt struct {
 	List    []Alias
@@ -919,6 +1026,11 @@ func (n ExportStmt) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n ExportStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // DirectivePrologueStmt is a string literal at the beginning of a function or module (usually "use strict").
 type DirectivePrologueStmt struct {
 	Value []byte
@@ -933,6 +1045,11 @@ func (n DirectivePrologueStmt) JS() string {
 	return string(n.Value)
 }
 
+// JSON converts the node back to valid JSON
+func (n DirectivePrologueStmt) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 func (n BlockStmt) stmtNode()             {}
 func (n EmptyStmt) stmtNode()             {}
 func (n ExprStmt) stmtNode()              {}
@@ -993,6 +1110,11 @@ func (n PropertyName) JS() string {
 	return n.String()
 }
 
+// JSON converts the node back to valid JSON
+func (n PropertyName) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // BindingArray is an array binding pattern.
 type BindingArray struct {
 	List []BindingElement
@@ -1034,6 +1156,11 @@ func (n BindingArray) JS() string {
 	return s + "]"
 }
 
+// JSON converts the node back to valid JSON
+func (n BindingArray) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // BindingObjectItem is a binding property.
 type BindingObjectItem struct {
 	Key   *PropertyName // can be nil
@@ -1061,6 +1188,11 @@ func (n BindingObjectItem) JS() string {
 	return " " + n.Value.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n BindingObjectItem) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // BindingObject is an object binding pattern.
 type BindingObject struct {
 	List []BindingObjectItem
@@ -1112,6 +1244,11 @@ func (n BindingObject) JS() string {
 	return s + " }"
 }
 
+// JSON converts the node back to valid JSON
+func (n BindingObject) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // BindingElement is a binding element.
 type BindingElement struct {
 	Binding IBinding // can be nil (in case of ellision)
@@ -1141,6 +1278,11 @@ func (n BindingElement) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n BindingElement) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 func (v *Var) bindingNode()          {}
 func (n BindingArray) bindingNode()  {}
 func (n BindingObject) bindingNode() {}
@@ -1173,6 +1315,11 @@ func (n VarDecl) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n VarDecl) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // Params is a list of parameters for functions, methods, and arrow function.
 type Params struct {
 	List []BindingElement
@@ -1214,6 +1361,11 @@ func (n Params) JS() string {
 	return s + ")"
 }
 
+// JSON converts the node back to valid JSON
+func (n Params) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // FuncDecl is an (async) (generator) function declaration or expression.
 type FuncDecl struct {
 	Async     bool
@@ -1256,6 +1408,11 @@ func (n FuncDecl) JS() string {
 	return s + " " + n.Params.JS() + " " + n.Body.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n FuncDecl) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // MethodDecl is a method definition in a class declaration.
 type MethodDecl struct {
 	Static    bool
@@ -1311,6 +1468,11 @@ func (n MethodDecl) JS() string {
 	return s[1:]
 }
 
+// JSON converts the node back to valid JSON
+func (n MethodDecl) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // FieldDefinition is a field definition in a class declaration.
 type FieldDefinition struct {
 	Name PropertyName
@@ -1334,6 +1496,11 @@ func (n FieldDefinition) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n FieldDefinition) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ClassDecl is a class declaration.
 type ClassDecl struct {
 	Name        *Var  // can be nil
@@ -1378,6 +1545,11 @@ func (n ClassDecl) JS() string {
 	return s + "}"
 }
 
+// JSON converts the node back to valid JSON
+func (n ClassDecl) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 func (n VarDecl) stmtNode()   {}
 func (n FuncDecl) stmtNode()  {}
 func (n ClassDecl) stmtNode() {}
@@ -1404,6 +1576,20 @@ func (n LiteralExpr) JS() string {
 	return string(n.Data)
 }
 
+// JSON converts the node back to valid JSON
+func (n LiteralExpr) JSON() (string, error) {
+	if n.TokenType == TrueToken || n.TokenType == FalseToken || n.TokenType == NullToken || n.TokenType == DecimalToken {
+		return string(n.Data), nil
+	} else if n.TokenType == StringToken {
+		if n.Data[0] == '\'' {
+			n.Data[0] = '"'
+			n.Data[len(n.Data)-1] = '"'
+		}
+		return string(n.Data), nil
+	}
+	return "", ErrInvalidJSON
+}
+
 // Element is an array literal element.
 type Element struct {
 	Value  IExpr // can be nil
@@ -1433,6 +1619,11 @@ func (n Element) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n Element) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ArrayExpr is an array literal.
 type ArrayExpr struct {
 	List []Element
@@ -1477,6 +1668,30 @@ func (n ArrayExpr) JS() string {
 	return s + "]"
 }
 
+// JSON converts the node back to valid JSON
+func (n ArrayExpr) JSON() (string, error) {
+	s := "["
+	for i, item := range n.List {
+		if i != 0 {
+			s += ", "
+		}
+		if item.Value != nil {
+			if item.Spread {
+				return "", ErrInvalidJSON
+			}
+			ss, err := item.Value.JSON()
+			if err != nil {
+				return "", err
+			}
+			s += ss
+		}
+	}
+	if 0 < len(n.List) && n.List[len(n.List)-1].Value == nil {
+		return "", ErrInvalidJSON
+	}
+	return s + "]", nil
+}
+
 // Property is a property definition in an object literal.
 type Property struct {
 	// either Name or Spread are set. When Spread is set then Value is AssignmentExpression
@@ -1520,6 +1735,29 @@ func (n Property) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n Property) JSON() (string, error) {
+	s := ""
+	if n.Name == nil || n.Name.Literal.TokenType != StringToken && n.Name.Literal.TokenType != IdentifierToken || n.Spread || n.Init != nil {
+		return "", ErrInvalidJSON
+	} else if n.Name.Literal.TokenType == IdentifierToken {
+		n.Name.Literal.TokenType = StringToken
+		n.Name.Literal.Data = append(append([]byte{'"'}, n.Name.Literal.Data...), '"')
+	}
+
+	ss, _ := n.Name.Literal.JSON()
+	s += ss + ": "
+
+	var err error
+	ss, err = n.Value.JSON()
+	if err != nil {
+		fmt.Println("b")
+		return "", err
+	}
+	s += ss
+	return s, nil
+}
+
 // ObjectExpr is an object literal.
 type ObjectExpr struct {
 	List []Property
@@ -1548,6 +1786,22 @@ func (n ObjectExpr) JS() string {
 	return s + "}"
 }
 
+// JSON converts the node back to valid JSON
+func (n ObjectExpr) JSON() (string, error) {
+	s := "{"
+	for i, item := range n.List {
+		if i != 0 {
+			s += ", "
+		}
+		ss, err := item.JSON()
+		if err != nil {
+			return "", err
+		}
+		s += ss
+	}
+	return s + "}", nil
+}
+
 // TemplatePart is a template head or middle.
 type TemplatePart struct {
 	Value []byte
@@ -1563,6 +1817,11 @@ func (n TemplatePart) JS() string {
 	return string(n.Value) + n.Expr.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n TemplatePart) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // TemplateExpr is a template literal or member/call expression, super property, or optional chain with template literal.
 type TemplateExpr struct {
 	Tag  IExpr // can be nil
@@ -1594,6 +1853,11 @@ func (n TemplateExpr) JS() string {
 	return s + string(n.Tail)
 }
 
+// JSON converts the node back to valid JSON
+func (n TemplateExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // GroupExpr is a parenthesized expression.
 type GroupExpr struct {
 	X IExpr
@@ -1608,6 +1872,11 @@ func (n GroupExpr) JS() string {
 	return "(" + n.X.JS() + ")"
 }
 
+// JSON converts the node back to valid JSON
+func (n GroupExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // IndexExpr is a member/call expression, super property, or optional chain with an index expression.
 type IndexExpr struct {
 	X    IExpr
@@ -1624,6 +1893,11 @@ func (n IndexExpr) JS() string {
 	return n.X.JS() + "[" + n.Y.JS() + "]"
 }
 
+// JSON converts the node back to valid JSON
+func (n IndexExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // DotExpr is a member/call expression, super property, or optional chain with a dot expression.
 type DotExpr struct {
 	X    IExpr
@@ -1640,6 +1914,11 @@ func (n DotExpr) JS() string {
 	return n.X.JS() + "." + n.Y.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n DotExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // NewTargetExpr is a new target meta property.
 type NewTargetExpr struct {
 }
@@ -1653,6 +1932,11 @@ func (n NewTargetExpr) JS() string {
 	return "new.target"
 }
 
+// JSON converts the node back to valid JSON
+func (n NewTargetExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ImportMetaExpr is a import meta meta property.
 type ImportMetaExpr struct {
 }
@@ -1666,6 +1950,11 @@ func (n ImportMetaExpr) JS() string {
 	return "import.meta"
 }
 
+// JSON converts the node back to valid JSON
+func (n ImportMetaExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 type Arg struct {
 	Value IExpr
 	Rest  bool
@@ -1688,6 +1977,11 @@ func (n Arg) JS() string {
 	return s + n.Value.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n Arg) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // Args is a list of arguments as used by new and call expressions.
 type Args struct {
 	List []Arg
@@ -1716,6 +2010,11 @@ func (n Args) JS() string {
 	return s
 }
 
+// JSON converts the node back to valid JSON
+func (n Args) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // NewExpr is a new expression or new member expression.
 type NewExpr struct {
 	X    IExpr
@@ -1739,6 +2038,11 @@ func (n NewExpr) JS() string {
 	return "new " + n.X.JS() + "()"
 }
 
+// JSON converts the node back to valid JSON
+func (n NewExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // CallExpr is a call expression.
 type CallExpr struct {
 	X    IExpr
@@ -1754,6 +2058,11 @@ func (n CallExpr) JS() string {
 	return n.X.JS() + "(" + n.Args.JS() + ")"
 }
 
+// JSON converts the node back to valid JSON
+func (n CallExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // OptChainExpr is an optional chain.
 type OptChainExpr struct {
 	X IExpr
@@ -1785,6 +2094,11 @@ func (n OptChainExpr) JS() string {
 	}
 }
 
+// JSON converts the node back to valid JSON
+func (n OptChainExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // UnaryExpr is an update or unary expression.
 type UnaryExpr struct {
 	Op TokenType
@@ -1810,6 +2124,11 @@ func (n UnaryExpr) JS() string {
 	return n.Op.String() + n.X.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n UnaryExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // BinaryExpr is a binary expression.
 type BinaryExpr struct {
 	Op   TokenType
@@ -1828,6 +2147,11 @@ func (n BinaryExpr) JS() string {
 	return n.X.JS() + " " + n.Op.String() + " " + n.Y.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n BinaryExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // CondExpr is a conditional expression.
 type CondExpr struct {
 	Cond, X, Y IExpr
@@ -1842,6 +2166,11 @@ func (n CondExpr) JS() string {
 	return n.Cond.JS() + " ? " + n.X.JS() + " : " + n.Y.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n CondExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // YieldExpr is a yield expression.
 type YieldExpr struct {
 	Generator bool
@@ -1871,6 +2200,11 @@ func (n YieldExpr) JS() string {
 	return s + " " + n.X.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n YieldExpr) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 // ArrowFunc is an (async) arrow function.
 type ArrowFunc struct {
 	Async  bool
@@ -1895,6 +2229,11 @@ func (n ArrowFunc) JS() string {
 	return s + n.Params.JS() + " => " + n.Body.JS()
 }
 
+// JSON converts the node back to valid JSON
+func (n ArrowFunc) JSON() (string, error) {
+	return "", ErrInvalidJSON
+}
+
 func (v *Var) exprNode()           {}
 func (n LiteralExpr) exprNode()    {}
 func (n ArrayExpr) exprNode()      {}
