@@ -9,8 +9,6 @@ import (
 	"github.com/tdewolff/parse/v2/buffer"
 )
 
-var evalBytes = []byte("eval")
-
 // Parser is the state for the parser.
 type Parser struct {
 	l   *Lexer
@@ -181,8 +179,18 @@ func (p *Parser) parseModule() (module BlockStmt) {
 		case ErrorToken:
 			return
 		case ImportToken:
-			importStmt := p.parseImportStmt()
-			module.List = append(module.List, &importStmt)
+			p.next()
+			if p.tt == OpenParenToken {
+				// could be an import call expression
+				left := &LiteralExpr{ImportToken, []byte("import")}
+				p.exprLevel++
+				suffix := p.parseExpressionSuffix(left, OpExpr, OpCall)
+				p.exprLevel--
+				module.List = append(module.List, &ExprStmt{suffix})
+			} else {
+				importStmt := p.parseImportStmt()
+				module.List = append(module.List, &importStmt)
+			}
 		case ExportToken:
 			exportStmt := p.parseExportStmt()
 			module.List = append(module.List, &exportStmt)
@@ -201,8 +209,7 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 
 	switch tt := p.tt; tt {
 	case OpenBraceToken:
-		blockStmt := p.parseBlockStmt("block statement")
-		stmt = &blockStmt
+		stmt = p.parseBlockStmt("block statement")
 	case ConstToken, VarToken:
 		if !allowDeclaration && tt == ConstToken {
 			p.fail("statement")
@@ -316,7 +323,7 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 			return
 		}
 
-		body := BlockStmt{}
+		body := &BlockStmt{}
 		parent := p.enterScope(&body.Scope, false)
 
 		var init IExpr
@@ -417,7 +424,8 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 			return
 		}
 
-		clauses := []CaseClause{}
+		switchStmt := &SwitchStmt{Init: init}
+		parent := p.enterScope(&switchStmt.Scope, false)
 		for {
 			if p.tt == ErrorToken {
 				p.fail("switch statement")
@@ -446,16 +454,16 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 			for p.tt != CaseToken && p.tt != DefaultToken && p.tt != CloseBraceToken && p.tt != ErrorToken {
 				stmts = append(stmts, p.parseStmt(true))
 			}
-			clauses = append(clauses, CaseClause{clause, list, stmts})
+			switchStmt.List = append(switchStmt.List, CaseClause{clause, list, stmts})
 		}
-		stmt = &SwitchStmt{init, clauses}
+		p.exitScope(parent)
+		stmt = switchStmt
 	case FunctionToken:
 		if !allowDeclaration {
 			p.fail("statement")
 			return
 		}
-		funcDecl := p.parseFuncDecl()
-		stmt = &funcDecl
+		stmt = p.parseFuncDecl()
 	case AsyncToken: // async function
 		if !allowDeclaration {
 			p.fail("statement")
@@ -464,8 +472,7 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 		async := p.data
 		p.next()
 		if p.tt == FunctionToken && !p.prevLT {
-			funcDecl := p.parseAsyncFuncDecl()
-			stmt = &funcDecl
+			stmt = p.parseAsyncFuncDecl()
 		} else {
 			// expression
 			stmt = &ExprStmt{p.parseAsyncExpression(OpExpr, async)}
@@ -479,8 +486,7 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 			p.fail("statement")
 			return
 		}
-		classDecl := p.parseClassDecl()
-		stmt = &classDecl
+		stmt = p.parseClassDecl()
 	case ThrowToken:
 		p.next()
 		var value IExpr
@@ -512,8 +518,7 @@ func (p *Parser) parseStmt(allowDeclaration bool) (stmt IStmt) {
 		}
 		if p.tt == FinallyToken {
 			p.next()
-			blockStmt := p.parseBlockStmt("try-finally statement")
-			finally = &blockStmt
+			finally = p.parseBlockStmt("try-finally statement")
 		}
 		stmt = &TryStmt{body, binding, catch, finally}
 	case DebuggerToken:
@@ -577,7 +582,8 @@ func (p *Parser) parseStmtList(in string) (list []IStmt) {
 	return
 }
 
-func (p *Parser) parseBlockStmt(in string) (blockStmt BlockStmt) {
+func (p *Parser) parseBlockStmt(in string) (blockStmt *BlockStmt) {
+	blockStmt = &BlockStmt{}
 	parent := p.enterScope(&blockStmt.Scope, false)
 	blockStmt.List = p.parseStmtList(in)
 	p.exitScope(parent)
@@ -585,8 +591,7 @@ func (p *Parser) parseBlockStmt(in string) (blockStmt BlockStmt) {
 }
 
 func (p *Parser) parseImportStmt() (importStmt ImportStmt) {
-	// assume we're at import
-	p.next()
+	// assume we're passed import
 	if p.tt == StringToken {
 		importStmt.Module = p.data
 		p.next()
@@ -724,38 +729,32 @@ func (p *Parser) parseExportStmt() (exportStmt ExportStmt) {
 		varDecl := p.parseVarDecl(tt)
 		exportStmt.Decl = &varDecl
 	} else if p.tt == FunctionToken {
-		funcDecl := p.parseFuncDecl()
-		exportStmt.Decl = &funcDecl
+		exportStmt.Decl = p.parseFuncDecl()
 	} else if p.tt == AsyncToken { // async function
 		p.next()
 		if p.tt != FunctionToken || p.prevLT {
 			p.fail("export statement", FunctionToken)
 			return
 		}
-		funcDecl := p.parseAsyncFuncDecl()
-		exportStmt.Decl = &funcDecl
+		exportStmt.Decl = p.parseAsyncFuncDecl()
 	} else if p.tt == ClassToken {
-		classDecl := p.parseClassDecl()
-		exportStmt.Decl = &classDecl
+		exportStmt.Decl = p.parseClassDecl()
 	} else if p.tt == DefaultToken {
 		exportStmt.Default = true
 		p.next()
 		if p.tt == FunctionToken {
-			funcDecl := p.parseFuncExpr()
-			exportStmt.Decl = &funcDecl
+			exportStmt.Decl = p.parseFuncExpr()
 		} else if p.tt == AsyncToken { // async function or async arrow function
 			async := p.data
 			p.next()
 			if p.tt == FunctionToken && !p.prevLT {
-				funcDecl := p.parseAsyncFuncExpr()
-				exportStmt.Decl = &funcDecl
+				exportStmt.Decl = p.parseAsyncFuncExpr()
 			} else {
 				// expression
 				exportStmt.Decl = p.parseAsyncExpression(OpExpr, async)
 			}
 		} else if p.tt == ClassToken {
-			classDecl := p.parseClassExpr()
-			exportStmt.Decl = &classDecl
+			exportStmt.Decl = p.parseClassExpr()
 		} else {
 			exportStmt.Decl = p.parseExpression(OpAssign)
 		}
@@ -832,25 +831,26 @@ func (p *Parser) parseFuncParams(in string) (params Params) {
 	return
 }
 
-func (p *Parser) parseFuncDecl() (funcDecl FuncDecl) {
+func (p *Parser) parseFuncDecl() (funcDecl *FuncDecl) {
 	return p.parseAnyFunc(false, false)
 }
 
-func (p *Parser) parseAsyncFuncDecl() (funcDecl FuncDecl) {
+func (p *Parser) parseAsyncFuncDecl() (funcDecl *FuncDecl) {
 	return p.parseAnyFunc(true, false)
 }
 
-func (p *Parser) parseFuncExpr() (funcDecl FuncDecl) {
+func (p *Parser) parseFuncExpr() (funcDecl *FuncDecl) {
 	return p.parseAnyFunc(false, true)
 }
 
-func (p *Parser) parseAsyncFuncExpr() (funcDecl FuncDecl) {
+func (p *Parser) parseAsyncFuncExpr() (funcDecl *FuncDecl) {
 	return p.parseAnyFunc(true, true)
 }
 
-func (p *Parser) parseAnyFunc(async, inExpr bool) (funcDecl FuncDecl) {
+func (p *Parser) parseAnyFunc(async, inExpr bool) (funcDecl *FuncDecl) {
 	// assume we're at function
 	p.next()
+	funcDecl = &FuncDecl{}
 	funcDecl.Async = async
 	funcDecl.Generator = p.tt == MulToken
 	if funcDecl.Generator {
@@ -891,17 +891,18 @@ func (p *Parser) parseAnyFunc(async, inExpr bool) (funcDecl FuncDecl) {
 	return
 }
 
-func (p *Parser) parseClassDecl() (classDecl ClassDecl) {
+func (p *Parser) parseClassDecl() (classDecl *ClassDecl) {
 	return p.parseAnyClass(false)
 }
 
-func (p *Parser) parseClassExpr() (classDecl ClassDecl) {
+func (p *Parser) parseClassExpr() (classDecl *ClassDecl) {
 	return p.parseAnyClass(true)
 }
 
-func (p *Parser) parseAnyClass(inExpr bool) (classDecl ClassDecl) {
+func (p *Parser) parseAnyClass(inExpr bool) (classDecl *ClassDecl) {
 	// assume we're at class
 	p.next()
+	classDecl = &ClassDecl{}
 	if IsIdentifier(p.tt) || p.tt == YieldToken || p.tt == AwaitToken {
 		if !inExpr {
 			var ok bool
@@ -940,7 +941,7 @@ func (p *Parser) parseAnyClass(inExpr bool) (classDecl ClassDecl) {
 		}
 
 		method, definition := p.parseClassElement()
-		if method.Name.IsSet() {
+		if method != nil {
 			classDecl.Methods = append(classDecl.Methods, method)
 		} else {
 			classDecl.Definitions = append(classDecl.Definitions, definition)
@@ -949,7 +950,8 @@ func (p *Parser) parseAnyClass(inExpr bool) (classDecl ClassDecl) {
 	return
 }
 
-func (p *Parser) parseClassElement() (method MethodDecl, definition FieldDefinition) {
+func (p *Parser) parseClassElement() (method *MethodDecl, definition FieldDefinition) {
+	method = &MethodDecl{}
 	var data []byte
 	if p.tt == StaticToken {
 		method.Static = true
@@ -1011,7 +1013,7 @@ func (p *Parser) parseClassElement() (method MethodDecl, definition FieldDefinit
 			p.next()
 			definition.Init = p.parseExpression(OpAssign)
 		}
-		method = MethodDecl{}
+		method = nil
 		return
 	}
 
@@ -1314,7 +1316,12 @@ func (p *Parser) parseObjectLiteral() (object ObjectExpr) {
 				method.Name.Literal.Data = parse.Copy(method.Name.Literal.Data) // copy so that renaming doesn't rename the key
 				property.Name = &method.Name                                    // set key explicitly so after renaming the original is still known
 				if p.assumeArrowFunc {
-					property.Value, _ = p.scope.Declare(ArgumentDecl, name) // cannot fail
+					var ok bool
+					property.Value, ok = p.scope.Declare(ArgumentDecl, name)
+					if !ok {
+						property.Value = p.scope.Use(name)
+						p.assumeArrowFunc = false
+					}
 				} else {
 					property.Value = p.scope.Use(name)
 				}
@@ -1383,8 +1390,9 @@ func (p *Parser) parseArguments() (args Args) {
 	return
 }
 
-func (p *Parser) parseAsyncArrowFunc() (arrowFunc ArrowFunc) {
+func (p *Parser) parseAsyncArrowFunc() (arrowFunc *ArrowFunc) {
 	// expect we're at Identifier or Yield or (
+	arrowFunc = &ArrowFunc{}
 	parent := p.enterScope(&arrowFunc.Body.Scope, true)
 	parentAsync, parentGenerator := p.async, p.generator
 	p.async, p.generator = true, false
@@ -1395,6 +1403,10 @@ func (p *Parser) parseAsyncArrowFunc() (arrowFunc ArrowFunc) {
 		arrowFunc.Params.List = []BindingElement{{Binding: ref}}
 	} else {
 		arrowFunc.Params = p.parseFuncParams("arrow function")
+
+		// could be CallExpression of: async(params)
+		if p.tt != ArrowToken {
+		}
 	}
 
 	arrowFunc.Async = true
@@ -1405,8 +1417,9 @@ func (p *Parser) parseAsyncArrowFunc() (arrowFunc ArrowFunc) {
 	return
 }
 
-func (p *Parser) parseIdentifierArrowFunc(v *Var) (arrowFunc ArrowFunc) {
+func (p *Parser) parseIdentifierArrowFunc(v *Var) (arrowFunc *ArrowFunc) {
 	// expect we're at =>
+	arrowFunc = &ArrowFunc{}
 	parent := p.enterScope(&arrowFunc.Body.Scope, true)
 	parentAsync, parentGenerator := p.async, p.generator
 	p.async, p.generator = false, false
@@ -1467,22 +1480,21 @@ func (p *Parser) parseAsyncExpression(prec OpPrec, async []byte) IExpr {
 	precLeft := OpPrimary
 	if !p.prevLT && p.tt == FunctionToken {
 		// primary expression
-		funcDecl := p.parseAsyncFuncExpr()
-		left = &funcDecl
+		left = p.parseAsyncFuncExpr()
 	} else if !p.prevLT && prec <= OpAssign && (p.tt == OpenParenToken || IsIdentifier(p.tt) || !p.generator && p.tt == YieldToken || p.tt == AwaitToken) {
 		// async arrow function expression
 		if p.tt == AwaitToken {
 			p.fail("arrow function")
 			return nil
+		} else if p.tt == OpenParenToken {
+			return p.parseParenthesizedExpressionOrArrowFunc(prec, async)
 		}
-		arrowFunc := p.parseAsyncArrowFunc()
-		left = &arrowFunc
+		left = p.parseAsyncArrowFunc()
 		precLeft = OpAssign
 	} else {
 		left = p.scope.Use(async)
 	}
-	left = p.parseExpressionSuffix(left, prec, precLeft)
-	return left
+	return p.parseExpressionSuffix(left, prec, precLeft)
 }
 
 // parseExpression parses an expression that has a precedence of prec or higher.
@@ -1549,7 +1561,7 @@ func (p *Parser) parseExpression(prec OpPrec) IExpr {
 			}
 			break
 		}
-		suffix := p.parseParenthesizedExpressionOrArrowFunc(prec)
+		suffix := p.parseParenthesizedExpressionOrArrowFunc(prec, nil)
 		p.exprLevel--
 		return suffix
 	case NotToken, BitNotToken, TypeofToken, VoidToken, DeleteToken:
@@ -1694,14 +1706,12 @@ func (p *Parser) parseExpression(prec OpPrec) IExpr {
 	case ClassToken:
 		parentInFor := p.inFor
 		p.inFor = false
-		classDecl := p.parseClassExpr()
-		left = &classDecl
+		left = p.parseClassExpr()
 		p.inFor = parentInFor
 	case FunctionToken:
 		parentInFor := p.inFor
 		p.inFor = false
-		funcDecl := p.parseFuncExpr()
-		left = &funcDecl
+		left = p.parseFuncExpr()
 		p.inFor = parentInFor
 	case TemplateToken, TemplateStartToken:
 		parentInFor := p.inFor
@@ -1719,7 +1729,12 @@ func (p *Parser) parseExpression(prec OpPrec) IExpr {
 }
 
 func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr {
-	for {
+	for i := 0; ; i++ {
+		if 1000 < p.exprLevel+i {
+			p.failMessage("too many nested expressions")
+			return nil
+		}
+
 		switch tt := p.tt; tt {
 		case EqToken, MulEqToken, DivEqToken, ModEqToken, ExpEqToken, AddEqToken, SubEqToken, LtLtEqToken, GtGtEqToken, GtGtGtEqToken, BitAndEqToken, BitXorEqToken, BitOrEqToken, AndEqToken, OrEqToken, NullishEqToken:
 			if OpAssign < prec {
@@ -2014,8 +2029,7 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 				return nil
 			}
 
-			arrowFunc := p.parseIdentifierArrowFunc(v)
-			left = &arrowFunc
+			left = p.parseIdentifierArrowFunc(v)
 			precLeft = OpAssign
 		default:
 			return left
@@ -2048,26 +2062,32 @@ func (p *Parser) parseAssignmentExpression() IExpr {
 	return p.parseExpression(OpAssign)
 }
 
-func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec) IExpr {
+func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec, async []byte) IExpr {
 	var left IExpr
 	precLeft := OpPrimary
 
 	// expect to be at (
 	p.next()
 
-	arrowFunc := ArrowFunc{}
+	isAsync := async != nil
+	arrowFunc := &ArrowFunc{}
 	parent := p.enterScope(&arrowFunc.Body.Scope, true)
 	parentAssumeArrowFunc, parentInFor := p.assumeArrowFunc, p.inFor
 	p.assumeArrowFunc, p.inFor = true, false
 
-	// parse a parenthesized expression but assume we might be parsing an arrow function. If this is really an arrow function, parsing as a parenthesized expression cannot fail as AssignmentExpression, ArrayLiteral, and ObjectLiteral are supersets of SingleNameBinding, ArrayBindingPattern, and ObjectBindingPattern respectively. Any identifier that would be a BindingIdentifier in case of an arrow function, will be added as such. If finally this is not an arrow function, we will demote those variables an undeclared and merge them with the parent scope.
+	// parse a parenthesized expression but assume we might be parsing an (async) arrow function. If this is really an arrow function, parsing as a parenthesized expression cannot fail as AssignmentExpression, ArrayLiteral, and ObjectLiteral are supersets of SingleNameBinding, ArrayBindingPattern, and ObjectBindingPattern respectively. Any identifier that would be a BindingIdentifier in case of an arrow function, will be added as such. If finally this is not an arrow function, we will demote those variables an undeclared and merge them with the parent scope.
 
 	var list []IExpr
 	var rest IExpr
 	for p.tt != CloseParenToken && p.tt != ErrorToken {
 		if p.tt == EllipsisToken && p.assumeArrowFunc {
 			p.next()
-			if p.isIdentifierReference(p.tt) {
+			if isAsync {
+				rest = p.parseAssignmentExpression()
+				if p.tt == CommaToken {
+					p.next()
+				}
+			} else if p.isIdentifierReference(p.tt) {
 				rest, _ = p.scope.Declare(ArgumentDecl, p.data) // cannot fail
 				p.next()
 			} else if p.tt == OpenBracketToken {
@@ -2099,22 +2119,23 @@ func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec) IExpr {
 
 	if isArrowFunc {
 		parentAsync, parentGenerator := p.async, p.generator
-		p.async, p.generator = false, false
+		p.async, p.generator = isAsync, false
 
 		// arrow function
 		arrowFunc.Params = Params{List: make([]BindingElement, len(list))}
 		for i, item := range list {
 			arrowFunc.Params.List[i] = p.exprToBindingElement(item) // can not fail when assumArrowFunc is set
 		}
+		arrowFunc.Async = isAsync
 		arrowFunc.Params.Rest = p.exprToBinding(rest)
 		arrowFunc.Body.List = p.parseArrowFuncBody()
 
 		p.async, p.generator = parentAsync, parentGenerator
 		p.exitScope(parent)
 
-		left = &arrowFunc
+		left = arrowFunc
 		precLeft = OpAssign
-	} else if len(list) == 0 || rest != nil {
+	} else if len(list) == 0 || !isAsync && rest != nil || isAsync && OpCall < prec {
 		p.fail("arrow function", ArrowToken)
 		return nil
 	} else {
@@ -2124,12 +2145,26 @@ func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec) IExpr {
 		// Here we move all declared ArgumentDecls (in case of an arrow function) to its parent scope as undeclared variables (identifiers used in a parenthesized expression).
 		arrowFunc.Body.Scope.UndeclareScope()
 
-		// parenthesized expression
-		left = list[0]
-		for _, item := range list[1:] {
-			left = &BinaryExpr{CommaToken, left, item}
+		if isAsync {
+			// call expression
+			args := Args{}
+			for _, item := range list {
+				args.List = append(args.List, Arg{Value: item, Rest: false})
+			}
+			if rest != nil {
+				args.List = append(args.List, Arg{Value: rest, Rest: true})
+			}
+			left = p.scope.Use(async)
+			left = &CallExpr{left, args}
+			precLeft = OpCall
+		} else {
+			// parenthesized expression
+			left = list[0]
+			for _, item := range list[1:] {
+				left = &BinaryExpr{CommaToken, left, item}
+			}
+			left = &GroupExpr{left}
 		}
-		left = &GroupExpr{left}
 	}
 	return p.parseExpressionSuffix(left, prec, precLeft)
 }
@@ -2162,7 +2197,9 @@ func (p *Parser) exprToBinding(expr IExpr) (binding IBinding) {
 			}
 			var bindingElement BindingElement
 			bindingElement.Binding = p.exprToBinding(item.Value)
-			if item.Init != nil {
+			if bindingElement.Binding == nil {
+				bindingElement = p.exprToBindingElement(item.Value)
+			} else if item.Init != nil {
 				bindingElement.Default = item.Init
 			}
 			bindingObject.List = append(bindingObject.List, BindingObjectItem{Key: item.Name, Value: bindingElement})
